// nesCE.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "nesCE.h"
#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>
#include <commdlg.h>
#include <stdlib.h>

#include "keyscan.h"

// GAPI include file
#include <gx.h>

extern "C" 
{
	#include "fceu\\m6502.h"
	#include "fceu\\svga.h"

	void PrintNonFatal(char *s);

	// handle to the main window
	HWND hWndNES;

	// color palette defined in PocketPCDriver.c
	unsigned short color_palette[256];

	// pointer to the bitmap screen produced by the emulator (8 bit color)
	byte *XBuf;
}

#define MAX_LOADSTRING 100

void OpenNewGame(HWND hWnd);
void DoPaint(HWND hWnd);

// Global Variables:
HINSTANCE			hInst;					// The current instance
HWND				hwndCB;					// The command bar handle

// GAPI stuff
GXDisplayProperties g_gxdp;
GXKeyList g_gxkl;


// directory for roms
char gDefaultDir[2048];

// rom file name to load
char szRomFileName[2048];

// indicates if the emulator thread is running or not
BOOL bThreadRunning = FALSE;

// command line arguments to send to the emulator
extern "C"
{
	extern char *argv[64];
	extern int argc;
}

static SHACTIVATEINFO s_sai;

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
BOOL				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About			(HWND, UINT, WPARAM, LPARAM);
HWND				CreateRpCommandBar(HWND);


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_NESCE);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	// clean up GAPI
	GXCloseDisplay();
	GXCloseInput();

	return msg.wParam;
}

//--------------------------------------------------------------------
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    It is important to call this function so that the application 
//    will get 'well formed' small icons associated with it.
//--------------------------------------------------------------------
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_NESCE));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//-----------------------------------------------------------------------------
//  FUNCTION: InitInstance(HANDLE, int)
//
//  PURPOSE: Saves instance handle and creates main window
//
//  COMMENTS:
//
//    In this function, we save the instance handle in a global variable and
//    create and display the main program window.
//-----------------------------------------------------------------------------
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd = NULL;
	TCHAR	szTitle[MAX_LOADSTRING];			// The title bar text
	TCHAR	szWindowClass[MAX_LOADSTRING];		// The window class name

	hInst = hInstance;		// Store instance handle in our global variable
	// Initialize global strings
	LoadString(hInstance, IDC_NESCE, szWindowClass, MAX_LOADSTRING);
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);

	//If it is already running, then focus on the window
	hWnd = FindWindow(szWindowClass, szTitle);	
	if (hWnd) 
	{
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));    
		return 0;
	} 

	MyRegisterClass(hInstance, szWindowClass);
	
	RECT	rect;
	GetClientRect(hWnd, &rect);
	
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	if (!hWnd)
	{	
		return FALSE;
	}
	//When the main window is created using CW_USEDEFAULT the height of the menubar (if one
	// is created is not taken into account). So we resize the window after creating it
	// if a menubar is present
	{
		RECT rc;
		GetWindowRect(hWnd, &rc);
		rc.bottom -= MENU_HEIGHT;
		if (hwndCB)
			MoveWindow(hWnd, rc.left, rc.top, rc.right, rc.bottom, FALSE);
	}


	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	gDefaultDir[0] = 0;

	// save the handle to this window for other functions to use
	hWndNES = hWnd;

	// set up GAPI
	// try opening the display for full screen access
	if(GXOpenDisplay(hWnd, GX_FULLSCREEN) == 0)
		return FALSE;  // error - we can't open the display, so exit

	// initialize the hardware buttons
	if(GXOpenInput() == 0)
		return FALSE;  // error - we can't open the buttons, so exit

	// get the display properties
	g_gxdp = GXGetDisplayProperties();

	// get information about the hardware keys and fill
	// the g_gxkl structure
	g_gxkl = GXGetDefaultKeys(GX_NORMALKEYS);

	return TRUE;
}

//------------------------------------------------------
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int wmId, wmEvent;
	TCHAR szHello[MAX_LOADSTRING];

	int keycode;


	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{	
				case IDM_HELP_ABOUT:
					DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				    break;
				case IDM_FILE_OPEN:
					OpenNewGame(hWnd);
					break;
				case IDOK:
					SendMessage(hWnd, WM_ACTIVATE, MAKEWPARAM(WA_INACTIVE, 0), (LPARAM)hWnd);
					SendMessage (hWnd, WM_CLOSE, 0, 0);
					break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			hwndCB = CreateRpCommandBar(hWnd);
			break;
		case WM_PAINT:
			DoPaint(hWnd);
			break; 
		case WM_DESTROY:
			CommandBar_Destroy(hwndCB);

			// destroy and free heap used throughout program
			PostQuitMessage(0);
			break;
		case WM_SETTINGCHANGE:
			SHHandleWMSettingChange(hWnd, wParam, lParam, &s_sai);
     		break;
		case WM_KEYDOWN:
			return 1;
			break;	

		case WM_KEYUP:
			return 1;
			break;

		case WM_KILLFOCUS:
			GXSuspend();
			break;

		case WM_SETFOCUS:
			GXResume();
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

HWND CreateRpCommandBar(HWND hwnd)
{
	SHMENUBARINFO mbi;

	memset(&mbi, 0, sizeof(SHMENUBARINFO));
	mbi.cbSize     = sizeof(SHMENUBARINFO);
	mbi.hwndParent = hwnd;
	mbi.nToolBarId = IDM_MENU;
	mbi.hInstRes   = hInst;
	mbi.nBmpId     = 0;
	mbi.cBmpImages = 0;

	if (!SHCreateMenuBar(&mbi)) 
		return NULL;

	return mbi.hwndMB;
}

// Mesage handler for the About box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	SHINITDLGINFO shidi;

	switch (message)
	{
		case WM_INITDIALOG:
			// Create a Done button and size it.  
			shidi.dwMask = SHIDIM_FLAGS;
			 shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hDlg;
			SHInitDialog(&shidi);
			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK) {
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


//-----------------------------------------------------------------------------------
// This is called when the user selects file->open from the main menu. Display a 
// file selection dialog, listing all the nes games. If the user selectes a game,
// save the path to the rom (in a global variable - I know... shouldnt use globals),
// and start a new emulation thread.
//----------------------------------------------------------------------------------- 
void OpenNewGame(HWND hWnd)
{
	TCHAR szFileName[256];
	TCHAR szDir[256];
	DWORD rc;
	HANDLE hThread;

	TCHAR filter[] = _T("NES Files(*.nes,*.nsf,*.fds)\0*.nes;*.nsf;*.fds\0All files (*.*)\0*.*\0");
	OPENFILENAME ofn;
	int i;

	// convert default rom directory from TCHAR to char
	for(i = 0; gDefaultDir[i] != 0; i++)
		szDir[i] = (TCHAR)gDefaultDir[i];
	gDefaultDir[i] = 0;

	// display a open file dialog
	memset(&ofn,0,sizeof(ofn));

	ofn.lStructSize		= sizeof(ofn);
	ofn.lpstrFilter		= filter;
	ofn.lpstrFile		= szFileName;
	ofn.nMaxFile		= 256;
	ofn.Flags			= OFN_EXPLORER | OFN_ENABLETEMPLATE;

	if(szDir[0]) 
		ofn.lpstrInitialDir = szDir;

	if(GetOpenFileName(&ofn))
	{
		// save the path to the rom file
		for(i = 0; szFileName[i] != 0; i++)
			szRomFileName[i] = (char)szFileName[i];

		szRomFileName[i] = 0;

		argc = 0;
		argv[argc++] = szRomFileName;
		argv[argc++] = szRomFileName;


		// save the new default rom directory
		for(i = 0; i < ofn.nFileOffset; i++)
			gDefaultDir[i] = (char)ofn.lpstrFile[i];
		gDefaultDir[i] = 0;

	
		// start a new emulation thread
		bThreadRunning = TRUE;

		// start a thread to play the game
		hThread = CreateThread(NULL, 
			0,		// stack size (ignored)
			(LPTHREAD_START_ROUTINE)fceumain,	// Thread function
			0,		// 32 bit parameter passed to thread
			0,		// Creation flags (0 = start thread immediately)
			&rc);	// Receives thread ID

		CloseHandle(hThread);
	}
}



void DoPaint(HWND hWnd)
{
	RECT rt;
	HDC hdcMem, hdc;
	HGDIOBJ hOldSel;
	PAINTSTRUCT ps;

	hdc = BeginPaint(hWnd, &ps);

	EndPaint(hWnd, &ps);
}


//-------------------------------------------------------------------------------------
// Blit the screen produced by the emulator to the screen. The output of the emulator
// is 272x256 pixels of 8 bits each (indexes into color_palette). We draw 240x240 of
// these pixels on the screen. We assume that the screen is 16 bit color arranged
// as rrrrrggggggbbbbb (the actual # of bits per color is determined in the SetPalette()
// function).
//-------------------------------------------------------------------------------------
void BlitScreen(void)
{
	// XBuf is the buffer with the screen data we need to blit to the screen.
	// It is arranged as scanlines of 272 bytes each (but only 256 are used per scanline), 
	// with a total of maxline scanlines (between 224 and 240).
	unsigned char *pbuf = XBuf;
	int x, y;
	int nPadDisplay;

	// get a pointer to the screen
	unsigned short *pscr = (unsigned short *)GXBeginDraw();
	if(pscr == NULL)
		return;

	// get the number of pixels padded at the end of each scanline for the screen
	nPadDisplay = (g_gxdp.cbyPitch >> 1) - 240;

	// start drawing 32 lines from top of display
	pscr += (240 + nPadDisplay) << 5;

	// chop off 8 pixels on both the left and the right side of the display
	// to make it fit.
	pbuf += 8; 

	for(y = 0; y < 240; y++)
	{
		for(x = 0; x < 240; x++)
		{
			*pscr = color_palette[*pbuf];
			pscr++;
			pbuf++;
		}
		pbuf += 32;
		pscr += nPadDisplay;
	}

	GXEndDraw();
}

